@@ -8,7 +8,7 @@ use Mojo::Redis::Cursor;
 use Mojo::Redis::Database;
 use Mojo::Redis::PubSub;
 
-our $VERSION = '3.03';
+our $VERSION = '3.04';
 
 $ENV{MOJO_REDIS_URL} ||= 'redis://localhost:6379';
 
@@ -88,9 +88,13 @@ Mojo::Redis - Redis driver based on Mojo::IOLoop
 
 =head1 SYNOPSIS
 
-  use Mojo::Redis;
+=head2 Blocking
 
+  use Mojo::Redis;
   my $redis = Mojo::Redis->new;
+  warn $redis->db->get("mykey");
+
+=head2 Promises
 
   $redis->db->get_p("mykey")->then(sub {
     print "mykey=$_[0]\n";
@@ -98,6 +102,22 @@ Mojo::Redis - Redis driver based on Mojo::IOLoop
     warn "Could not fetch mykey: $_[0]";
   })->wait;
 
+=head2 Pipelining
+
+Pipelining is built into the API by sending a lot of commands and then use
+L<Mojo::Promise/all> to wait for all the responses.
+
+  Mojo::Promise->all(
+    $db->set_p($key, 10),
+    $db->incrby_p($key, 9),
+    $db->incr_p($key),
+    $db->get_p($key),
+    $db->incr_p($key),
+    $db->get_p($key),
+  )->then(sub {
+    @res = map {@$_} @_;
+  })->wait;
+
 =head1 DESCRIPTION
 
 L<Mojo::Redis> is a Redis driver that use the L<Mojo::IOLoop>, which makes it
