@@ -2,37 +2,43 @@ package Mojo::Redis::Database;
 use Mojo::Base -base;
 
 our @BASIC_COMMANDS = (
-  'append',            'bitcount',         'bitfield',     'bitop',
-  'bitpos',            'echo',             'eval',         'evalsha',
-  'decr',              'decrby',           'del',          'dump',
-  'exists',            'expire',           'expireat',     'geoadd',
-  'geohash',           'geopos',           'geodist',      'georadius',
-  'georadiusbymember', 'get',              'getbit',       'getrange',
-  'getset',            'hdel',             'hexists',      'hget',
-  'hgetall',           'hincrby',          'hincrbyfloat', 'hkeys',
-  'hlen',              'hmget',            'hmset',        'hset',
-  'hsetnx',            'hstrlen',          'hvals',        'incr',
-  'incrby',            'incrbyfloat',      'keys',         'lindex',
-  'linsert',           'llen',             'lpop',         'lpush',
-  'lpushx',            'lrange',           'lrem',         'lset',
-  'ltrim',             'mget',             'move',         'mset',
-  'msetnx',            'object',           'persist',      'pexpire',
-  'pexpireat',         'pttl',             'pfadd',        'pfcount',
-  'pfmerge',           'ping',             'psetex',       'publish',
-  'randomkey',         'rename',           'renamenx',     'rpop',
-  'rpoplpush',         'rpush',            'rpushx',       'restore',
-  'sadd',              'scard',            'script',       'sdiff',
-  'sdiffstore',        'set',              'setbit',       'setex',
-  'setnx',             'setrange',         'sinter',       'sinterstore',
-  'sismember',         'smembers',         'smove',        'sort',
-  'spop',              'srandmember',      'srem',         'strlen',
-  'sunion',            'sunionstore',      'touch',        'ttl',
-  'type',              'unlink',           'zadd',         'zcard',
-  'zcount',            'zincrby',          'zinterstore',  'zlexcount',
-  'zpopmax',           'zpopmin',          'zrange',       'zrangebylex',
-  'zrangebyscore',     'zrank',            'zrem',         'zremrangebylex',
-  'zremrangebyrank',   'zremrangebyscore', 'zrevrange',    'zrevrangebylex',
-  'zrevrangebyscore',  'zrevrank',         'zscore',       'zunionstore',
+  'append',           'bgrewriteaof',      'bgsave',      'bitcount',
+  'bitfield',         'bitop',             'bitpos',      'client',
+  'config',           'command',           'dbsize',      'debug',
+  'decr',             'decrby',            'del',         'dump',
+  'echo',             'eval',              'evalsha',     'exists',
+  'expire',           'expireat',          'flushall',    'flushdb',
+  'geoadd',           'geohash',           'geopos',      'geodist',
+  'georadius',        'georadiusbymember', 'get',         'getbit',
+  'getrange',         'getset',            'hdel',        'hexists',
+  'hget',             'hgetall',           'hincrby',     'hincrbyfloat',
+  'hkeys',            'hlen',              'hmget',       'hmset',
+  'hset',             'hsetnx',            'hstrlen',     'hvals',
+  'info',             'incr',              'incrby',      'incrbyfloat',
+  'keys',             'lastsave',          'lindex',      'linsert',
+  'llen',             'lpop',              'lpush',       'lpushx',
+  'lrange',           'lrem',              'lset',        'ltrim',
+  'memory',           'mget',              'move',        'mset',
+  'msetnx',           'object',            'persist',     'pexpire',
+  'pexpireat',        'pttl',              'pfadd',       'pfcount',
+  'pfmerge',          'ping',              'psetex',      'publish',
+  'randomkey',        'rename',            'renamenx',    'role',
+  'rpop',             'rpoplpush',         'rpush',       'rpushx',
+  'restore',          'sadd',              'save',        'scard',
+  'script',           'sdiff',             'sdiffstore',  'set',
+  'setbit',           'setex',             'setnx',       'setrange',
+  'sinter',           'sinterstore',       'sismember',   'slaveof',
+  'slowlog',          'smembers',          'smove',       'sort',
+  'spop',             'srandmember',       'srem',        'strlen',
+  'sunion',           'sunionstore',       'time',        'touch',
+  'ttl',              'type',              'unlink',      'xadd',
+  'xrange',           'xrevrange',         'xlen',        'xread',
+  'xreadgroup',       'xpending',          'zadd',        'zcard',
+  'zcount',           'zincrby',           'zinterstore', 'zlexcount',
+  'zpopmax',          'zpopmin',           'zrange',      'zrangebylex',
+  'zrangebyscore',    'zrank',             'zrem',        'zremrangebylex',
+  'zremrangebyrank',  'zremrangebyscore',  'zrevrange',   'zrevrangebylex',
+  'zrevrangebyscore', 'zrevrank',          'zscore',      'zunionstore',
 );
 
 our @BLOCKING_COMMANDS = ('blpop', 'brpop', 'brpoplpush', 'bzpopmax', 'bzpopmin');
@@ -147,6 +153,11 @@ sub _process_sdiff             { +[@_] }
 sub _process_smembers          { +[@_] }
 sub _process_sort              { +[@_] }
 sub _process_sunion            { +[@_] }
+sub _process_xrange            { +[@_] }
+sub _process_xread             { +[@_] }
+sub _process_xreadgroup        { +[@_] }
+sub _process_xrevrange         { +[@_] }
+sub _process_xpending          { +[@_] }
 sub _process_zrange            { +[@_] }
 sub _process_zrangebylex       { +[@_] }
 sub _process_zrangebyscore     { +[@_] }
@@ -219,6 +230,26 @@ Append a value to a key.
 
 See L<https://redis.io/commands/append> for more information.
 
+=head2 bgrewriteaof
+
+  @res     = $self->bgrewriteaof;
+  $self    = $self->bgrewriteaof(sub { my ($self, @res) = @_ });
+  $promise = $self->bgrewriteaof_p;
+
+Asynchronously rewrite the append-only file.
+
+See L<https://redis.io/commands/bgrewriteaof> for more information.
+
+=head2 bgsave
+
+  @res     = $self->bgsave;
+  $self    = $self->bgsave(sub { my ($self, @res) = @_ });
+  $promise = $self->bgsave_p;
+
+Asynchronously save the dataset to disk.
+
+See L<https://redis.io/commands/bgsave> for more information.
+
 =head2 bitcount
 
   @res     = $self->bitcount($key, [start end]);
@@ -309,6 +340,64 @@ Remove and return the member with the lowest score from one or more sorted sets,
 
 See L<https://redis.io/commands/bzpopmin> for more information.
 
+=head2 client
+
+  @res     = $self->command(@args);
+  $self    = $self->command(@args, sub { my ($self, @res) = @_ });
+  $promise = $self->command_p(@args);
+
+Run a "CLIENT" command on the server. C<@args> can be:
+
+=over 2
+
+=item * KILL [ip:port] [ID client-id] [TYPE normal|master|slave|pubsub] [ADDR ip:port] [SKIPME yes/no]
+
+=item * LIST
+
+=item * GETNAME
+
+=item * PAUSE timeout
+
+=item * REPLY [ON|OFF|SKIP]
+
+=item * SETNAME connection-name
+
+=back
+
+See L<https://redis.io/commands#server> for more information.
+
+=head2 command
+
+  @res     = $self->command(@args);
+  $self    = $self->command(@args, sub { my ($self, @res) = @_ });
+  $promise = $self->command_p(@args);
+
+Get array of Redis command details.
+
+=over 2
+
+=item * empty list
+
+=item * COUNT
+
+=item * GETKEYS
+
+=item * INFO command-name [command-name]
+
+=back
+
+See L<https://redis.io/commands/command> for more information.
+
+=head2 dbsize
+
+  @res     = $self->dbsize;
+  $self    = $self->dbsize(sub { my ($self, @res) = @_ });
+  $promise = $self->dbsize_p;
+
+Return the number of keys in the selected database.
+
+See L<https://redis.io/commands/dbsize> for more information.
+
 =head2 decr
 
   @res     = $self->decr($key);
@@ -437,6 +526,26 @@ Set the expiration for a key as a UNIX timestamp.
 
 See L<https://redis.io/commands/expireat> for more information.
 
+=head2 flushall
+
+  @res     = $self->flushall([ASYNC]);
+  $self    = $self->flushall([ASYNC], sub { my ($self, @res) = @_ });
+  $promise = $self->flushall_p([ASYNC]);
+
+Remove all keys from all databases.
+
+See L<https://redis.io/commands/flushall> for more information.
+
+=head2 flushdb
+
+  @res     = $self->flushdb([ASYNC]);
+  $self    = $self->flushdb([ASYNC], sub { my ($self, @res) = @_ });
+  $promise = $self->flushdb_p([ASYNC]);
+
+Remove all keys from the current database.
+
+See L<https://redis.io/commands/flushdb> for more information.
+
 =head2 geoadd
 
   @res     = $self->geoadd($key, $longitude latitude member [longitude latitude member ...]);
@@ -677,6 +786,16 @@ Get all the values in a hash.
 
 See L<https://redis.io/commands/hvals> for more information.
 
+=head2 info
+
+  @res     = $self->info([section]);
+  $self    = $self->info([section], sub { my ($self, @res) = @_ });
+  $promise = $self->info_p([section]);
+
+Get information and statistics about the server.
+
+See L<https://redis.io/commands/info> for more information.
+
 =head2 incr
 
   @res     = $self->incr($key);
@@ -717,6 +836,16 @@ Find all keys matching the given pattern.
 
 See L<https://redis.io/commands/keys> for more information.
 
+=head2 lastsave
+
+  @res     = $self->lastsave;
+  $self    = $self->lastsave(sub { my ($self, @res) = @_ });
+  $promise = $self->lastsave_p;
+
+Get the UNIX time stamp of the last successful save to disk.
+
+See L<https://redis.io/commands/lastsave> for more information.
+
 =head2 lindex
 
   @res     = $self->lindex($key, $index);
@@ -1013,6 +1142,16 @@ Rename a key, only if the new key does not exist.
 
 See L<https://redis.io/commands/renamenx> for more information.
 
+=head2 role
+
+  @res     = $self->role;
+  $self    = $self->role(sub { my ($self, @res) = @_ });
+  $promise = $self->role_p;
+
+Return the role of the instance in the context of replication.
+
+See L<https://redis.io/commands/role> for more information.
+
 =head2 rpop
 
   @res     = $self->rpop($key);
@@ -1073,6 +1212,16 @@ Add one or more members to a set.
 
 See L<https://redis.io/commands/sadd> for more information.
 
+=head2 save
+
+  @res     = $self->save;
+  $self    = $self->save(sub { my ($self, @res) = @_ });
+  $promise = $self->save_p;
+
+Synchronously save the dataset to disk.
+
+See L<https://redis.io/commands/save> for more information.
+
 =head2 scard
 
   @res     = $self->scard($key);
@@ -1197,6 +1346,26 @@ Determine if a given value is a member of a set.
 
 See L<https://redis.io/commands/sismember> for more information.
 
+=head2 slaveof
+
+  @res     = $self->slaveof($host, $port);
+  $self    = $self->slaveof($host, $port, sub { my ($self, @res) = @_ });
+  $promise = $self->slaveof_p($host, $port);
+
+Make the server a slave of another instance, or promote it as master.
+
+See L<https://redis.io/commands/slaveof> for more information.
+
+=head2 slowlog
+
+  @res     = $self->slowlog($subcommand, [argument]);
+  $self    = $self->slowlog($subcommand, [argument], sub { my ($self, @res) = @_ });
+  $promise = $self->slowlog_p($subcommand, [argument]);
+
+Manages the Redis slow queries log.
+
+See L<https://redis.io/commands/slowlog> for more information.
+
 =head2 smembers
 
   @res     = $self->smembers($key);
@@ -1287,6 +1456,16 @@ Add multiple sets and store the resulting set in a key.
 
 See L<https://redis.io/commands/sunionstore> for more information.
 
+=head2 time
+
+  @res     = $self->time;
+  $self    = $self->time(sub { my ($self, @res) = @_ });
+  $promise = $self->time_p;
+
+Return the current server time.
+
+See L<https://redis.io/commands/time> for more information.
+
 =head2 touch
 
   @res     = $self->touch($key [key ...]);
@@ -1329,9 +1508,9 @@ See L<https://redis.io/commands/unlink> for more information.
 
 =head2 unwatch
 
-  @res     = $self->unwatch();
-  $self    = $self->unwatch(, sub { my ($self, @res) = @_ });
-  $promise = $self->unwatch_p();
+  @res     = $self->unwatch;
+  $self    = $self->unwatch(sub { my ($self, @res) = @_ });
+  $promise = $self->unwatch_p;
 
 Forget about all watched keys.
 
@@ -1347,6 +1526,76 @@ Watch the given keys to determine execution of the MULTI/EXEC block.
 
 See L<https://redis.io/commands/watch> for more information.
 
+=head2 xadd
+
+  @res     = $self->xadd($key, $ID, $field string [field string ...]);
+  $self    = $self->xadd($key, $ID, $field string [field string ...], sub { my ($self, @res) = @_ });
+  $promise = $self->xadd_p($key, $ID, $field string [field string ...]);
+
+Appends a new entry to a stream.
+
+See L<https://redis.io/commands/xadd> for more information.
+
+=head2 xlen
+
+  @res     = $self->xlen($key);
+  $self    = $self->xlen($key, sub { my ($self, @res) = @_ });
+  $promise = $self->xlen_p($key);
+
+Return the number of entires in a stream.
+
+See L<https://redis.io/commands/xlen> for more information.
+
+=head2 xpending
+
+  @res     = $self->xpending($key, $group, [start end count], [consumer]);
+  $self    = $self->xpending($key, $group, [start end count], [consumer], sub { my ($self, @res) = @_ });
+  $promise = $self->xpending_p($key, $group, [start end count], [consumer]);
+
+Return information and entries from a stream consumer group pending entries list, that are messages fetched but never acknowledged.
+
+See L<https://redis.io/commands/xpending> for more information.
+
+=head2 xrange
+
+  @res     = $self->xrange($key, $start, $end, [COUNT count]);
+  $self    = $self->xrange($key, $start, $end, [COUNT count], sub { my ($self, @res) = @_ });
+  $promise = $self->xrange_p($key, $start, $end, [COUNT count]);
+
+Return a range of elements in a stream, with IDs matching the specified IDs interval.
+
+See L<https://redis.io/commands/xrange> for more information.
+
+=head2 xread
+
+  @res     = $self->xread([COUNT count], [BLOCK milliseconds], $STREAMS, $key [key ...], $ID [ID ...]);
+  $self    = $self->xread([COUNT count], [BLOCK milliseconds], $STREAMS, $key [key ...], $ID [ID ...], sub { my ($self, @res) = @_ });
+  $promise = $self->xread_p([COUNT count], [BLOCK milliseconds], $STREAMS, $key [key ...], $ID [ID ...]);
+
+Return never seen elements in multiple streams, with IDs greater than the ones reported by the caller for each stream. Can block.
+
+See L<https://redis.io/commands/xread> for more information.
+
+=head2 xreadgroup
+
+  @res     = $self->xreadgroup($GROUP group consumer, [COUNT count], [BLOCK milliseconds], $STREAMS, $key [key ...], $ID [ID ...]);
+  $self    = $self->xreadgroup($GROUP group consumer, [COUNT count], [BLOCK milliseconds], $STREAMS, $key [key ...], $ID [ID ...], sub { my ($self, @res) = @_ });
+  $promise = $self->xreadgroup_p($GROUP group consumer, [COUNT count], [BLOCK milliseconds], $STREAMS, $key [key ...], $ID [ID ...]);
+
+Return new entries from a stream using a consumer group, or access the history of the pending entries for a given consumer. Can block.
+
+See L<https://redis.io/commands/xreadgroup> for more information.
+
+=head2 xrevrange
+
+  @res     = $self->xrevrange($key, $end, $start, [COUNT count]);
+  $self    = $self->xrevrange($key, $end, $start, [COUNT count], sub { my ($self, @res) = @_ });
+  $promise = $self->xrevrange_p($key, $end, $start, [COUNT count]);
+
+Return a range of elements in a stream, with IDs matching the specified IDs interval, in reverse order (from greater to smaller IDs) compared to XRANGE.
+
+See L<https://redis.io/commands/xrevrange> for more information.
+
 =head2 zadd
 
   @res     = $self->zadd($key, [NX|XX], [CH], [INCR], $score member [score member ...]);
